#include "SimpleAnalysisFramework/AnalysisClass.h"
//#include "SimpleAnalysis/AnalysisClass.h"
#include "SimpleAnalysisFramework/BDT.h"
#include "TSystem.h"
#include "TMath.h"

DefineAnalysis(ANA-SUSY-2019-23)

// tcMeT analysis (Run2 2015+2016+2017+2018 data) [SUSY-2019-23]
// Support Doc: ATL-COM-PHYS-2021-130
//              https://cds.cern.ch/record/2759797
// @author : A.Lopez Solis
//


void tcMeT_2018::Init()
{
  //** tcMET analysis (tc+MET channel)

  //signal regions

  addRegions({"SRAll"});
  addRegions({"SRA","SRA_bin1","SRA_bin2"});
  addRegions({"SRB0","SRB0_bin1","SRB0_bin2","SRB0_bin3"});
  addRegions({"SRB1","SRB1_bin1","SRB1_bin2","SRB1_bin3","SRB1_bin4"});
  addRegions({"SRC","SRC_bin1","SRC_bin2","SRC_bin3","SRC_bin4","SRC_bin5","SRC_bin6"});

  //control regions
  addRegions({"SRA_CRZ","SRA_CRst"}); // Control regions SRA
  
  //Control regions SRB

  addRegions({"SRB0_CRZ","SRB0_CRst"}); // Control regions SRB
  addRegions({"SRB1_CRZ","SRB1_CRst"}); // Control regions SRB

  //Control regions SRC
  addRegions({"SRC_CRZ","SRC_CRtt","SRC_CRW"}); // Control regions SRC

  //validation regions

  //cutFlow regions

  // Book 1/2D histograms

  addHistogram("MET",40,0,2000);
  addHistogram("metSigST",20,0,40);
  addHistogram("pT_1jet",20,20,2000);
  addHistogram("MET_prime",20,0,2000);
  addHistogram("HT",20,0,2000);
  addHistogram("MT",20,0,340);
  addHistogram("meff",20,0,2000);
  addHistogram("mct2b",20,0,1000);
  addHistogram("mbb",20,0,1000);
  addHistogram("mctjj",20,0,1000);
  addHistogram("dphimin2",20,0,3.14);
  addHistogram("dphimin3",20,0,3.14);
  addHistogram("dphimin4",20,0,3.14);
  addHistogram("abs_dPhi_b1b2",20,0,3.14);
  addHistogram("deltaPlus",20,0,3.14);
  addHistogram("deltaMinus",20,0,3.14);
  addHistogram("HTRatio",20,0,1.2);
  addHistogram("ht3",20,0,1.2);
  addHistogram("A",20,0,1.2);
  addHistogram("mlbmin",20,0,1000);

}

bool tcMETSUSY1Selection(AnalysisEvent& event, const tcMeT_2018& tcmet) {
  //Preselection applied in SUSY1
  //Taken from https://gitlab.cern.ch/atlas/athena/blob/21.2/PhysicsAnalysis/DerivationFramework/DerivationFrameworkSUSY/share/SUSY1.py#L201
  //Although the analysis uses pflow jets, SUSY1 filters on calorimeter jets. So do NOT apply PFlowJet id.

  auto jets40_SUSY1 = event.getJets(40,2.8);
  float HT_SUSY1 = tcmet.sumObjectsPt(jets40_SUSY1,99);
  bool passHT_SUSY1 = HT_SUSY1 > 150;

  auto electrons_SUSY1 = event.getElectrons(20, 2.6, ELooseLH);
  auto electrons100_SUSY1 = tcmet.filterObjects(electrons_SUSY1,100);
  bool passElecCut_SUSY1 = (electrons100_SUSY1.size() > 0 || electrons_SUSY1.size() > 1);

  //DFCommonMuonsPreselection corresonds to VeryLoose (3)
  auto muons_SUSY1 = event.getMuons(20,2.6,MuVeryLoose);
  auto muons100_SUSY1 = tcmet.filterObjects(muons_SUSY1,100);
  bool passMuonCut_SUSY1 = (muons100_SUSY1.size() > 0 || muons_SUSY1.size() > 1);

  //We use DFCommonPhotons, which I believe is defined here:
  //https://acode-browser1.usatlas.bnl.gov/lxr/source/athena/PhysicsAnalysis/DerivationFramework/DerivationFrameworkEGamma/src/PhotonsDirectionTool.cxx?v=21.2
  //and indicates no ID level is required. So choose the loosest selection, PhotonLoose.
  auto photons_SUSY1 = event.getPhotons(50,2.6,PhotonLoose);
  auto photons100_SUSY1 = tcmet.filterObjects(photons_SUSY1,100);
  bool passPhotonCut_SUSY1 = (photons100_SUSY1.size() > 0 || photons_SUSY1.size() > 0);

  return (passHT_SUSY1 || passElecCut_SUSY1 || passMuonCut_SUSY1 || passPhotonCut_SUSY1);
}

void tcMETCommonVariables(std::map<std::string,float>& common_vars,AnalysisEvent& event, const tcMeT_2018& tcmet,std::map<std::string,AnalysisObjects> objects){

  // Initial cut for ttbar and single-top sliced samples
  common_vars["gen_met"]   = event.getGenMET();
  common_vars["mc_channel"] = event.getMCNumber();

  // Reading key objects
  auto baseline_electrons = objects["BaseElectrons"];
  auto baseline_muons = objects["BaseMuons"];
  auto sig_electrons = objects["SignalElectrons"];
  auto sig_muons = objects["SignalMuons"];
  auto jets = objects["jets"];
  auto jets35 = objects["jets35"];
  auto nonbjets = objects["nonbjets"];
  auto bjets = objects["bjets"];
  auto cjets = objects["cjets"];
  auto fatjets = objects["fatjets"];
  auto fatjetbtag = objects["fatjetsbtagged"];
  auto fatjetbtagnctag = objects["fatjetsbtaggednonctagged"];
  auto fatjetttag = objects["fatjetsttagged"];
  auto topjets = objects["topjets"];
  auto bjets35 = objects["bjets35"];
  auto vertices = objects["vertices"];
  auto metVec     = event.getMET();
  auto JetLep = objects["JetLep"];

  auto signalLeptons = sig_electrons + sig_muons;
  tcmet.sortObjectsByPt(signalLeptons);
  auto baselineLeptons = baseline_electrons + baseline_muons;
  tcmet.sortObjectsByPt(baselineLeptons);

  // Number of objects
  common_vars["nsigEl"] = sig_electrons.size();
  common_vars["nbaseEl"] = baseline_electrons.size();
  common_vars["nEl"] = sig_electrons.size();


  common_vars["nsigMu"] = sig_muons.size();
  common_vars["nbaseMu"] = baseline_muons.size();
  common_vars["nMu"] = sig_muons.size();

  common_vars["nsigLep"] = sig_electrons.size() + sig_muons.size();
  common_vars["nbaselineLep"] = baseline_electrons.size() + baseline_muons.size();

  common_vars["njets"] = jets.size();
  common_vars["njets35"] = jets35.size() ;
  common_vars["nbjets"] = bjets.size();
  common_vars["nbjets35"] = bjets35.size() ;
  common_vars["ncjets"] = cjets.size();

  common_vars["nDNNTop_fatjet_kt10"] = topjets.size();
  common_vars["nfatjet"] = fatjets.size();
  common_vars["nfatjetbtag"] = fatjetbtag.size();
  common_vars["nfatjetbtagnctag"] = fatjetbtagnctag.size();
  common_vars["nfatjetttag"] = fatjetttag.size();

  common_vars["pass1etriggers"] = true;
  common_vars["pass1mutriggers"] = true;
  common_vars["passMETtriggers"] = true;

  //Lepton variables

  common_vars["pT_1baseLep"] = baselineLeptons.size() >= 1 ? baselineLeptons[0].Pt() : -999.0;
  common_vars["pT_2baseLep"] = baselineLeptons.size() >= 2 ? baselineLeptons[1].Pt() : -999.0;
  common_vars["ptLep1"] = signalLeptons.size() >= 1 ? signalLeptons[0].Pt() : -999.0;
  common_vars["ptLep2"] = signalLeptons.size() >= 2 ? signalLeptons[1].Pt() : -999.0;
  common_vars["chargeLep1"] = signalLeptons.size() >= 1 ? signalLeptons[0].charge() : -999.0;
  common_vars["chargeLep2"] = signalLeptons.size() >= 2 ? signalLeptons[1].charge() : -999.0;

  float mll = 0.0;
  if ( (baseline_electrons.size() >= 2 && 0 == baseline_muons.size()) || (0 == baseline_electrons.size() && baseline_muons.size() >= 2) ){
    mll = (baselineLeptons[0] + baselineLeptons[1]).M();
  }
  else if ( baselineLeptons.size() >= 2){
    //Taken from TDatabasePDG (cannot use this because ATLAS does not install it on cvmfs)
    float ZMass = 91.1870;
    float ZWidth = 2.48009;

    float min_ZMass = 1000000.0;
    for (unsigned int countA = 0; countA < baselineLeptons.size(); countA++){
      auto leptonA = baselineLeptons[countA];
      for (unsigned int countB = countA + 1; countB < baselineLeptons.size(); countB++){
        auto leptonB = baselineLeptons[countB];
        if (leptonA.charge() != leptonB.charge()){
          float candMass = (leptonA + leptonB).M();
          if ( fabs(candMass - ZMass) < fabs(min_ZMass - candMass) ) min_ZMass = candMass;
        }
      }
    }

    if (min_ZMass >= ZMass+4*ZWidth || min_ZMass < ZMass-4*ZWidth) min_ZMass = 0.0;
    mll = min_ZMass;

  }
  common_vars["mll"] = mll;


  // Jet variables
  common_vars["pT_1jet"] = jets.size() >= 1 ? jets[0].Pt() : -999.0;
  common_vars["pT_2jet"] = jets.size() >= 2 ? jets[1].Pt() : -999.0;
  common_vars["pT_3jet"] = jets.size() >= 3 ? jets[2].Pt() : -999.0;
  common_vars["pT_4jet"] = jets.size() >= 4 ? jets[3].Pt() : -999.0;
  common_vars["pT_1bjet"] = bjets.size() >= 1 ? bjets[0].Pt() : -999.0;
  common_vars["pT_2bjet"] = bjets.size() >= 2 ? bjets[1].Pt() : -999.0;
  common_vars["pT_3bjet"] = bjets.size() >= 3 ? bjets[2].Pt() : -999.0;
  common_vars["pT_4bjet"] = bjets.size() >= 4 ? bjets[3].Pt() : -999.0;
  common_vars["eta_1jet"] = jets.size() >= 1 ? jets[0].Eta() : -999.0;
  common_vars["eta_2jet"] = jets.size() >= 2 ? jets[1].Eta() : -999.0;
  common_vars["eta_3jet"] = jets.size() >= 3 ? jets[2].Eta() : -999.0;
  common_vars["eta_4jet"] = jets.size() >= 4 ? jets[3].Eta() : -999.0;
  common_vars["eta_1bjet"] = bjets.size() >= 1 ? bjets[0].Eta() : -999.0;
  common_vars["eta_2bjet"] = bjets.size() >= 2 ? bjets[1].Eta() : -999.0;
  common_vars["eta_3bjet"] = bjets.size() >= 3 ? bjets[2].Eta() : -999.0;
  common_vars["eta_4bjet"] = bjets.size() >= 4 ? bjets[3].Eta() : -999.0;
  common_vars["pT_1cjet"] = cjets.size() >= 1 ? cjets[0].Pt() : -999.0;
  common_vars["pT_2cjet"] = cjets.size() >= 2 ? cjets[1].Pt() : -999.0;
  common_vars["pT_3cjet"] = cjets.size() >= 3 ? cjets[2].Pt() : -999.0;
  common_vars["pT_4cjet"] = cjets.size() >= 4 ? cjets[3].Pt() : -999.0;
  common_vars["eta_1cjet"] = cjets.size() >= 1 ? cjets[0].Eta() : -999.0;
  common_vars["eta_2cjet"] = cjets.size() >= 2 ? cjets[1].Eta() : -999.0;
  common_vars["eta_3cjet"] = cjets.size() >= 3 ? cjets[2].Eta() : -999.0;
  common_vars["eta_4cjet"] = cjets.size() >= 4 ? cjets[3].Eta() : -999.0;
  common_vars["phi_1jet"] = jets.size() >= 1 ? jets[0].Phi() : -999.0;
  common_vars["phi_2jet"] = jets.size() >= 2 ? jets[1].Phi() : -999.0;
  common_vars["phi_3jet"] = jets.size() >= 3 ? jets[2].Phi() : -999.0;
  common_vars["phi_4jet"] = jets.size() >= 4 ? jets[3].Phi() : -999.0;
  common_vars["phi_1bjet"] = bjets.size() >= 1 ? bjets[0].Phi() : -999.0;
  common_vars["phi_2bjet"] = bjets.size() >= 2 ? bjets[1].Phi() : -999.0;
  common_vars["phi_3bjet"] = bjets.size() >= 3 ? bjets[2].Phi() : -999.0;
  common_vars["phi_4bjet"] = bjets.size() >= 4 ? bjets[3].Phi() : -999.0;
  common_vars["phi_1cjet"] = cjets.size() >= 1 ? cjets[0].Phi() : -999.0;
  common_vars["phi_2cjet"] = cjets.size() >= 2 ? cjets[1].Phi() : -999.0;
  common_vars["phi_3cjet"] = cjets.size() >= 3 ? cjets[2].Phi() : -999.0;
  common_vars["phi_4cjet"] = cjets.size() >= 4 ? cjets[3].Phi() : -999.0;

  common_vars["nlightjets"] = nonbjets.size();
  common_vars["pT_1nonbjet"] = nonbjets.size() >= 1 ? nonbjets[0].Pt() : -999.0;
  common_vars["eta_1nonbjet"] = nonbjets.size() >= 1 ? nonbjets[0].Eta() : -999.0;
  common_vars["phi_1nonbjet"] = nonbjets.size() >= 1 ? nonbjets[0].Phi() : -999.0;


  //auto isBTagJet1 = 
  common_vars["isBTagJet1"] = jets.size() >=1 && jets[0].pass(BTag77DL1r) ? 1.0 : 0.0;
  common_vars["isBTagJet2"] = jets.size() >=2 && jets[1].pass(BTag77DL1r) ? 1.0 : 0.0;
  common_vars["isBTagJet3"] = jets.size() >=3 && jets[2].pass(BTag77DL1r) ? 1.0 : 0.0;
  common_vars["isBTagJet4"] = jets.size() >=4 && jets[3].pass(BTag77DL1r) ? 1.0 : 0.0;
  common_vars["isBTagJet5"] = jets.size() >=5 && jets[4].pass(BTag77DL1r) ? 1.0 : 0.0;
  common_vars["isBTagJet6"] = jets.size() >=6 && jets[5].pass(BTag77DL1r) ? 1.0 : 0.0;
  common_vars["isCTagJet1"] = jets.size() >=1 && jets[0].pass(TrueCJet) ? 1.0 : 0.0; //jets->nonbjets for orig
  common_vars["isCTagJet2"] = jets.size() >=2 && jets[1].pass(TrueCJet) ? 1.0 : 0.0;
  common_vars["isCTagJet3"] = jets.size() >=3 && jets[2].pass(TrueCJet) ? 1.0 : 0.0;
  common_vars["isCTagJet4"] = jets.size() >=4 && jets[3].pass(TrueCJet) ? 1.0 : 0.0;
  common_vars["isCTagJet5"] = jets.size() >=5 && jets[4].pass(TrueCJet) ? 1.0 : 0.0;
  common_vars["isCTagJet6"] = jets.size() >=6 && jets[5].pass(TrueCJet) ? 1.0 : 0.0;

  common_vars["isBTagJetLep1"] = JetLep.size() >=1 && JetLep[0].pass(BTag77DL1r) ? 1.0 : 0.0;
  common_vars["isBTagJetLep2"] = JetLep.size() >=2 && JetLep[1].pass(BTag77DL1r) ? 1.0 : 0.0;
  common_vars["isBTagJetLep3"] = JetLep.size() >=3 && JetLep[2].pass(BTag77DL1r) ? 1.0 : 0.0;
  common_vars["isBTagJetLep4"] = JetLep.size() >=4 && JetLep[3].pass(BTag77DL1r) ? 1.0 : 0.0;
  common_vars["isBTagJetLep5"] = JetLep.size() >=5 && JetLep[4].pass(BTag77DL1r) ? 1.0 : 0.0;
  common_vars["isBTagJetLep6"] = JetLep.size() >=6 && JetLep[5].pass(BTag77DL1r) ? 1.0 : 0.0;
  common_vars["isCTagJetLep1"] = JetLep.size() >=1 && JetLep[0].pass(TrueCJet) ? 1.0 : 0.0;
  common_vars["isCTagJetLep2"] = JetLep.size() >=2 && JetLep[1].pass(TrueCJet) ? 1.0 : 0.0;
  common_vars["isCTagJetLep3"] = JetLep.size() >=3 && JetLep[2].pass(TrueCJet) ? 1.0 : 0.0;
  common_vars["isCTagJetLep4"] = JetLep.size() >=4 && JetLep[3].pass(TrueCJet) ? 1.0 : 0.0;
  common_vars["isCTagJetLep5"] = JetLep.size() >=5 && JetLep[4].pass(TrueCJet) ? 1.0 : 0.0;
  common_vars["isCTagJetLep6"] = JetLep.size() >=6 && JetLep[5].pass(TrueCJet) ? 1.0 : 0.0;


  // MET variables
  common_vars["metOrig"] = metVec.Et();
  common_vars["metPhi"] = metVec.Phi();
  common_vars["metSigST"] = event.getMETSignificance();
  common_vars["met"] = common_vars["metOrig"];
  common_vars["metTrack"] = common_vars["metOrig"];

  if( common_vars["nsigLep"] == 2 ){
    TVector2 lep1;
    lep1.SetMagPhi(signalLeptons[0].Pt(),signalLeptons[0].Phi() );
    TVector2 lep2;
    lep2.SetMagPhi(signalLeptons[1].Pt(),signalLeptons[1].Phi() );
    TVector2 metvector;
    metvector.SetMagPhi(metVec.Et(),metVec.Phi() );
    common_vars["met"] = (metvector+lep2+lep1).Mod();
    common_vars["met_prime_phi"] = (metvector+lep2+lep1).Phi();
    
    std::vector<float> dphimet_prime;
    for (unsigned int i = 0 ; i <= 3 ; ++i)
      dphimet_prime.push_back( jets.size() >= i+1 ? fabs( TVector2::Phi_mpi_pi((metvector+lep2+lep1).Phi() - jets[i].Phi() )) : 999 );
    
    common_vars["dphimin2_prime"] = *std::min_element( dphimet_prime.begin(), dphimet_prime.begin()+2 );
    common_vars["dphimin3_prime"] = *std::min_element( dphimet_prime.begin(), dphimet_prime.begin()+3 );
    common_vars["dphimin4_prime"] = *std::min_element( dphimet_prime.begin(), dphimet_prime.begin()+3 );
    
  }

  //Anti-QCD variables
  std::vector<float> dphimet;
  for (unsigned int i = 0 ; i <= 3 ; ++i)
    dphimet.push_back( jets.size() >= i+1 ? fabs( TVector2::Phi_mpi_pi(metVec.Phi() - jets[i].Phi() )) : 999 );

  common_vars["dphimin2"] = *std::min_element( dphimet.begin(), dphimet.begin()+2 );
  common_vars["dphimin3"] = *std::min_element( dphimet.begin(), dphimet.begin()+3 );
  common_vars["dphimin4"] = *std::min_element( dphimet.begin(), dphimet.begin()+4 );


  // Angular variables
  common_vars["dRb1b2"] = common_vars["nbjets"] > 1 ? bjets[0].DeltaR(bjets[1]) : -99. ;
  common_vars["dRj1j2"] = common_vars["njets"] > 1 ? jets[0].DeltaR(jets[1]) : -99. ;

  // General variables

  common_vars["HT"] = tcmet.sumObjectsPt(jets,99);
  common_vars["ht3"] = 0;
  for( unsigned int i = 0 ; i < jets.size() ; ++i){
    if(i>=3)
      common_vars["ht3"] += jets[i].Pt();
  }
  common_vars["HTRatio"] = common_vars["pT_1jet"]/common_vars["HT"];
  common_vars["meff"] = common_vars["met"] + common_vars["HT"];
  common_vars["MT"] =  common_vars["nsigLep"] >= 1 ? tcmet.calcMT(signalLeptons[0], metVec) : 0.;

  common_vars["mlbmin"] = -999.0;
  if(common_vars["nsigLep"]){
    if(common_vars["nbjets"]>1)
      common_vars["mlbmin"] = std::min( (signalLeptons[0] + bjets[0]).M(), (signalLeptons[0] + bjets[1]).M());
    else if(common_vars["nbjets"]>0)
     common_vars["mlbmin"] = (signalLeptons[0] + bjets[0]).M();
  }

  std::vector<float> mtbs;
  for (unsigned int i = 0 ; i < bjets.size() ; ++i)
    mtbs.push_back( sqrt(2*(metVec.Et())*bjets[i].Et() - 2*((metVec.Px())*bjets[i].Px() + (metVec.Py())*bjets[i].Py()) ) );

  common_vars["MTbMin"] =  mtbs.size() >= 1 ? *std::min_element( mtbs.begin(), mtbs.end() ) : -999.0;
  common_vars["MTbMax"] =  mtbs.size() >= 1 ? *std::max_element( mtbs.begin(), mtbs.end() ) : -999.0;


  std::vector<float> mtcs;
  for (unsigned int i = 0 ; i < cjets.size() ; ++i)
    mtcs.push_back( sqrt(2*(metVec.Et())*cjets[i].Et() - 2*((metVec.Px())*cjets[i].Px() + (metVec.Py())*cjets[i].Py()) ) );

  common_vars["MTcMin"] = mtcs.size() >= 1 ? *std::min_element( mtcs.begin(), mtcs.end() ) : -999.0;
  common_vars["MTcMax"] = mtcs.size() >= 1 ? *std::max_element( mtcs.begin(), mtcs.end() ) : -999.0;


  float mtjclose = -999.0;
  float phi = 999 ;
  for (unsigned int i = 0 ; i < jets.size() ; ++i){
    if(phi > fabs(TVector2::Phi_mpi_pi(jets[i].Phi() - metVec.Phi()))){
      phi = fabs(TVector2::Phi_mpi_pi(jets[i].Phi() - metVec.Phi()));
      mtjclose=sqrt(2*(metVec.Et())*jets[i].Et() - 2*((metVec.Px())*jets[i].Px() + (metVec.Py())*jets[i].Py()) );
    }
  }

  common_vars["MTjMinClosePhi"] = mtjclose;


  // High level variables for the tcMET analysis

  //common_vars["MT2bInFatJet"] = common_vars["nfatjetbtag"] >= 1 && common_vars["ncjets"] >= 1 ?tcmet.calcMT2(fatjetbtag[0],cjets[0],metVec) : -999.0 ;
  common_vars["MT2bInFatJet"] = common_vars["nfatjetbtagnctag"] >= 1 && common_vars["ncjets"] >= 1 ? tcmet.calcMT2(fatjetbtagnctag[0],cjets[0],metVec) : -999.0 ;

  //Variables for NN, some duplicates
  common_vars["ptJet1"] = jets.size() >= 1 ? jets[0].Pt() : -999.0;
  common_vars["ptJet2"] = jets.size() >= 2 ? jets[1].Pt() : -999.0;
  common_vars["ptJet3"] = jets.size() >= 3 ? jets[2].Pt() : -999.0;
  common_vars["ptJet4"] = jets.size() >= 4 ? jets[3].Pt() : -999.0;
  common_vars["ptJet5"] = jets.size() >= 5 ? jets[4].Pt() : -999.0;
  common_vars["ptJet6"] = jets.size() >= 6 ? jets[5].Pt() : -999.0;
  common_vars["etaJet1"] = jets.size() >= 1 ? jets[0].Eta() : -999.0;
  common_vars["etaJet2"] = jets.size() >= 2 ? jets[1].Eta() : -999.0;
  common_vars["etaJet3"] = jets.size() >= 3 ? jets[2].Eta() : -999.0;
  common_vars["etaJet4"] = jets.size() >= 4 ? jets[3].Eta() : -999.0;
  common_vars["etaJet5"] = jets.size() >= 5 ? jets[4].Eta() : -999.0;
  common_vars["etaJet6"] = jets.size() >= 6 ? jets[5].Eta() : -999.0;
  common_vars["dPhiJet1"] = jets.size() >= 1 ? fabs( TVector2::Phi_mpi_pi((metVec).Phi() - jets[0].Phi() )) : -999.0;
  common_vars["dPhiJet2"] = jets.size() >= 2 ? fabs( TVector2::Phi_mpi_pi((metVec).Phi() - jets[1].Phi() )) : -999.0;
  common_vars["dPhiJet3"] = jets.size() >= 3 ? fabs( TVector2::Phi_mpi_pi((metVec).Phi() - jets[2].Phi() )) : -999.0;
  common_vars["dPhiJet4"] = jets.size() >= 4 ? fabs( TVector2::Phi_mpi_pi((metVec).Phi() - jets[3].Phi() )) : -999.0;
  common_vars["dPhiJet5"] = jets.size() >= 5 ? fabs( TVector2::Phi_mpi_pi((metVec).Phi() - jets[4].Phi() )) : -999.0;
  common_vars["dPhiJet6"] = jets.size() >= 6 ? fabs( TVector2::Phi_mpi_pi((metVec).Phi() - jets[5].Phi() )) : -999.0;
  common_vars["ptBJet1"] = bjets.size() >= 1 ? bjets[0].Pt() : -999.0;
  common_vars["ptBJet2"] = bjets.size() >= 2 ? bjets[1].Pt() : -999.0;
  common_vars["ptCJet1"] = cjets.size() >= 1 ? cjets[0].Pt() : -999.0;
  common_vars["ptCJet2"] = cjets.size() >= 2 ? cjets[1].Pt() : -999.0;
  common_vars["dPhiBJet1"] = bjets.size() >= 1 ? fabs( TVector2::Phi_mpi_pi((metVec).Phi() - bjets[0].Phi() )) : -999.0;
  common_vars["dPhiBJet2"] = bjets.size() >= 2 ? fabs( TVector2::Phi_mpi_pi((metVec).Phi() - bjets[1].Phi() )) : -999.0;
  common_vars["dPhiCJet1"] = cjets.size() >= 1 ? fabs( TVector2::Phi_mpi_pi((metVec).Phi() - cjets[0].Phi() )) : -999.0;
  common_vars["dPhiCJet2"] = cjets.size() >= 2 ? fabs( TVector2::Phi_mpi_pi((metVec).Phi() - cjets[1].Phi() )) : -999.0;
  common_vars["etaBJet1"] = bjets.size() >= 1 ? bjets[0].Eta() : -999.0;
  common_vars["etaBJet2"] = bjets.size() >= 2 ? bjets[1].Eta() : -999.0;
  common_vars["etaCJet1"] = cjets.size() >= 1 ? cjets[0].Eta() : -999.0;
  common_vars["etaCJet2"] = cjets.size() >= 2 ? cjets[1].Eta() : -999.0;
  common_vars["NJet"] = jets.size();
  common_vars["NBTag"] = bjets.size();
  common_vars["NCTag"] = cjets.size();
  common_vars["flavJet1"] = (common_vars["isBTagJet1"] + 2*common_vars["isCTagJet1"]) == 3 ? 1 : common_vars["isBTagJet1"] + 2*common_vars["isCTagJet1"];
  common_vars["flavJet2"] = (common_vars["isBTagJet2"] + 2*common_vars["isCTagJet2"]) == 3 ? 1 : common_vars["isBTagJet2"] + 2*common_vars["isCTagJet2"];
  common_vars["flavJet3"] = (common_vars["isBTagJet3"] + 2*common_vars["isCTagJet3"]) == 3 ? 1 : common_vars["isBTagJet3"] + 2*common_vars["isCTagJet3"];
  common_vars["flavJet4"] = (common_vars["isBTagJet4"] + 2*common_vars["isCTagJet4"]) == 3 ? 1 : common_vars["isBTagJet4"] + 2*common_vars["isCTagJet4"];
  common_vars["flavJet5"] = (common_vars["isBTagJet5"] + 2*common_vars["isCTagJet5"]) == 3 ? 1 : common_vars["isBTagJet5"] + 2*common_vars["isCTagJet5"];
  common_vars["flavJet6"] = (common_vars["isBTagJet6"] + 2*common_vars["isCTagJet6"]) == 3 ? 1 : common_vars["isBTagJet6"] + 2*common_vars["isCTagJet6"];
  common_vars["NJetLep"] = JetLep.size();
  common_vars["flavJetLep1"] = (common_vars["isBTagJetLep1"] + 2*common_vars["isCTagJetLep1"]) == 3 ? 1 : common_vars["isBTagJetLep1"] + 2*common_vars["isCTagJetLep1"];
  common_vars["flavJetLep2"] = (common_vars["isBTagJetLep2"] + 2*common_vars["isCTagJetLep2"]) == 3 ? 1 : common_vars["isBTagJetLep2"] + 2*common_vars["isCTagJetLep2"];
  common_vars["flavJetLep3"] = (common_vars["isBTagJetLep3"] + 2*common_vars["isCTagJetLep3"]) == 3 ? 1 : common_vars["isBTagJetLep3"] + 2*common_vars["isCTagJetLep3"];
  common_vars["flavJetLep4"] = (common_vars["isBTagJetLep4"] + 2*common_vars["isCTagJetLep4"]) == 3 ? 1 : common_vars["isBTagJetLep4"] + 2*common_vars["isCTagJetLep4"];
  common_vars["flavJetLep5"] = (common_vars["isBTagJetLep5"] + 2*common_vars["isCTagJetLep5"]) == 3 ? 1 : common_vars["isBTagJetLep5"] + 2*common_vars["isCTagJetLep5"];
  common_vars["flavJetLep6"] = (common_vars["isBTagJetLep6"] + 2*common_vars["isCTagJetLep6"]) == 3 ? 1 : common_vars["isBTagJetLep6"] + 2*common_vars["isCTagJetLep6"];
  common_vars["ptJetLep1"] = JetLep.size() >= 1 ? JetLep[0].Pt() : -999.0;
  common_vars["ptJetLep2"] = JetLep.size() >= 2 ? JetLep[1].Pt() : -999.0;
  common_vars["ptJetLep3"] = JetLep.size() >= 3 ? JetLep[2].Pt() : -999.0;
  common_vars["ptJetLep4"] = JetLep.size() >= 4 ? JetLep[3].Pt() : -999.0;
  common_vars["ptJetLep5"] = JetLep.size() >= 5 ? JetLep[4].Pt() : -999.0;
  common_vars["ptJetLep6"] = JetLep.size() >= 6 ? JetLep[5].Pt() : -999.0;
  common_vars["etaJetLep1"] = JetLep.size() >= 1 ? JetLep[0].Eta() : -999.0;
  common_vars["etaJetLep2"] = JetLep.size() >= 2 ? JetLep[1].Eta() : -999.0;
  common_vars["etaJetLep3"] = JetLep.size() >= 3 ? JetLep[2].Eta() : -999.0;
  common_vars["etaJetLep4"] = JetLep.size() >= 4 ? JetLep[3].Eta() : -999.0;
  common_vars["etaJetLep5"] = JetLep.size() >= 5 ? JetLep[4].Eta() : -999.0;
  common_vars["etaJetLep6"] = JetLep.size() >= 6 ? JetLep[5].Eta() : -999.0;
  common_vars["dPhiJetLep1"] = JetLep.size() >= 1 ? fabs( TVector2::Phi_mpi_pi((metVec).Phi() - JetLep[0].Phi() )) : -999.0;
  common_vars["dPhiJetLep2"] = JetLep.size() >= 2 ? fabs( TVector2::Phi_mpi_pi((metVec).Phi() - JetLep[1].Phi() )) : -999.0;
  common_vars["dPhiJetLep3"] = JetLep.size() >= 3 ? fabs( TVector2::Phi_mpi_pi((metVec).Phi() - JetLep[2].Phi() )) : -999.0;
  common_vars["dPhiJetLep4"] = JetLep.size() >= 4 ? fabs( TVector2::Phi_mpi_pi((metVec).Phi() - JetLep[3].Phi() )) : -999.0;
  common_vars["dPhiJetLep5"] = JetLep.size() >= 5 ? fabs( TVector2::Phi_mpi_pi((metVec).Phi() - JetLep[4].Phi() )) : -999.0;
  common_vars["dPhiJetLep6"] = JetLep.size() >= 6 ? fabs( TVector2::Phi_mpi_pi((metVec).Phi() - JetLep[5].Phi() )) : -999.0;
  
}

void tcMETCommonSelections(std::map<std::string,bool>& commonSel, std::map<std::string,float>& common_vars) {

  // Reading key objects
  auto nbaseLep = int(common_vars["nbaselineLep"]);
  auto nsigLep = int(common_vars["nsigLep"]);
  auto nbaseEl = int(common_vars["nbaseEl"]);
  auto nbaseMu = int(common_vars["nbaseMu"]);
  auto pT_1baseLep = common_vars["pT_1baseLep"];
  auto pT_2baseLep = common_vars["pT_2baseLep"];
  auto mt = common_vars["MT"];
  auto met = common_vars["metOrig"];
  auto mll = common_vars["mll"];
  auto njets = int(common_vars["njets"]);
  auto ncjets = int(common_vars["ncjets"]);
  auto nbjets = int(common_vars["nbjets"]);
  auto dphimin4 = common_vars["dphimin4"];
  auto dphimin3 = common_vars["dphimin3"];
  auto dphimin2 = common_vars["dphimin2"];
  auto met_prime = common_vars["met"];
  auto pT_1jet = common_vars["pT_1jet"];
  auto nsbv_loose = common_vars["nsbv_loose"];

  //define 0,1,2-lepton cuts
  commonSel["pass0Lep"] = (0 == nbaseLep);
  commonSel["pass1Lep"] = (1 == nbaseLep && 1 == nsigLep && pT_1baseLep > 20 && mt > 20);
  commonSel["pass2Lep"] = (2 == nbaseLep && 2 == nsigLep && ( (2 == nbaseEl && 0 == nbaseMu) || (0 == nbaseEl && 2 == nbaseMu) ) && pT_1baseLep > 27 && pT_2baseLep > 20 && mt > 20 && (mll > 76 && mll < 106) && met < 150);

  // Define basic jet preselection
  commonSel["pre-1b-1c-3j"]=met_prime > 250 && dphimin4 > 0.4 && njets >= 3 && nbjets >=1 && ncjets >= 1;
  commonSel["pre-1b-1c-2j"]=met_prime > 250 && dphimin3 > 0.3 && njets >= 2 && nbjets >=1 && ncjets >= 1;

  commonSel["pre-1b-1c-3j-0l"] = commonSel["pre-1b-1c-3j"] && commonSel["pass0Lep"];
  commonSel["pre-1b-1c-3j-1l"] = commonSel["pre-1b-1c-3j"] && commonSel["pass1Lep"];
  commonSel["pre-1b-1c-3j-2l"] = commonSel["pre-1b-1c-3j"] && commonSel["pass2Lep"];


  commonSel["pre-1b-1c-2j-0l"] = commonSel["pre-1b-1c-2j"] && commonSel["pass0Lep"];
  commonSel["pre-1b-1c-2j-1l"] = commonSel["pre-1b-1c-2j"] && commonSel["pass1Lep"];
  commonSel["pre-1b-1c-2j-2l"] = commonSel["pre-1b-1c-2j"] && commonSel["pass2Lep"];


}

void tcMETSRBSelections(std::map<std::string,float>& cvars,std::map<std::string,bool>& csel){

  auto njets = cvars["njets"];
  auto pT_1jet = cvars["pT_1jet"];
  auto pT_2jet = cvars["pT_2jet"];
  auto meff = cvars["meff"];
  auto nbjets = cvars["nbjets"];
  auto ncjets = cvars["ncjets"];

  bool SRB0 =  njets >= 5 && pT_2jet >= 100.0 && cvars["MTjMinClosePhi"] >= 100 && cvars["pT_4jet"] >= 50.0 && cvars["MTbMin"] > 200.0 && cvars["MTbMax"] >= 200 && cvars["MTbMax"] <= 700 && cvars["MTcMin"] >= 150 && cvars["MTcMax"]>=400 && cvars["pT_1cjet"] >= 100.0 && cvars["nDNNTop_fatjet_kt10"] == 0 && cvars["metSigST"] > 10 && cvars["MT2bInFatJet"] >= 150 && cvars["metOrig"]>= 300 && (cvars["isBTagJet1"] == 1 || cvars["isCTagJet1"] == 1) ;
  bool SRB1 =  cvars["MTjMinClosePhi"] >= 100 && cvars["MTbMin"] > 200.0 && cvars["MTcMin"] >= 300 && cvars["pT_1cjet"] >= 100.0 && cvars["pT_1bjet"] >= 100.0 && cvars["nDNNTop_fatjet_kt10"] >= 1 && cvars["metSigST"] > 17 && cvars["MT2bInFatJet"] >= 200 && cvars["MT2bInFatJet"] < 450 && (cvars["isBTagJet1"] == 1 || cvars["isCTagJet1"] == 1) ;


  csel["SRB0"] = csel["pre-1b-1c-3j-0l"] && SRB0;
  csel["SRB0_bin1"] = csel["SRB0"] && cvars["MTjMinClosePhi"] <= 150;
  csel["SRB0_bin2"] = csel["SRB0"] && cvars["MTjMinClosePhi"] > 150 && cvars["MTjMinClosePhi"] < 400;
  csel["SRB0_bin3"] = csel["SRB0"] && cvars["MTjMinClosePhi"] >= 400;

  csel["SRB1"] = csel["pre-1b-1c-3j-0l"] && SRB1;
  csel["SRB1_bin1"] = csel["SRB1"] && cvars["MTjMinClosePhi"] <= 150;
  csel["SRB1_bin2"] = csel["SRB1"] && cvars["MTjMinClosePhi"] > 150 && cvars["MTjMinClosePhi"] < 300;
  csel["SRB1_bin3"] = csel["SRB1"] && cvars["MTjMinClosePhi"] >= 300 && cvars["MTjMinClosePhi"] < 500;
  csel["SRB1_bin4"] = csel["SRB1"] && cvars["MTjMinClosePhi"] >=500;

}


void tcMETSRASelections(std::map<std::string,float>& cvars,std::map<std::string,bool>& csel){

  auto njets = cvars["njets"];
  auto pT_1jet = cvars["pT_1jet"];
  auto pT_2jet = cvars["pT_2jet"];
  auto meff = cvars["meff"];
  auto nbjets = cvars["nbjets"];
  auto ncjets = cvars["ncjets"];
  auto ntjets = cvars["nDNNTop_fatjet_kt10"];//Tristan


  bool SRA = njets >=3 && nbjets >= 1 && ncjets >= 1 && cvars["nDNNTop_fatjet_kt10"] >= 1 && /*pT_1jet > 50 &&*/ cvars["metSigST"] > 18 && cvars["MT2bInFatJet"] >= 450  && cvars["MTjMinClosePhi"] >= 100 && cvars["MTbMin"] >= 200.0 && cvars["MTcMin"] >= 200.0 && (cvars["isBTagJet1"] == 1 || cvars["isCTagJet1"] == 1);
  csel["SRA"] = csel["pre-1b-1c-3j-0l"] && SRA;
  csel["SRA_bin1"] = csel["SRA"] && cvars["MT2bInFatJet"] <= 575;
  csel["SRA_bin2"] = csel["SRA"] && cvars["MT2bInFatJet"] > 575;

}


void tcMETSRCSelections(std::map<std::string,float>& cvars,std::map<std::string,bool>& csel){

  auto njets = cvars["njets"];
  auto pT_1jet = cvars["pT_1jet"];
  auto pT_2jet = cvars["pT_2jet"];
  auto meff = cvars["meff"];
  auto nbjets = cvars["nbjets"];
  auto ncjets = cvars["ncjets"];

  bool SRC = cvars["metSigST"] > 6 && cvars["isBTagJet1"] == 0 && cvars["isCTagJet1"] == 0 && cvars["MTcMin"] >= 100.0 && pT_1jet >= 100.0 && cvars["pT_4jet"] >= 30.0;
  csel["SRC"]=csel["pre-1b-1c-2j-0l"] && cvars["njets"] >= 3 && SRC && cvars["dRj1j2"] >= 1.0 ;
  csel["SRC_bin1"] = csel["SRC"] && cvars["meff"] >= 750 && cvars["meff"] < 1000.0;
  csel["SRC_bin2"] = csel["SRC"] && cvars["meff"] >= 1000.0 && cvars["meff"] < 1250.0;
  csel["SRC_bin3"] = csel["SRC"] && cvars["meff"] >= 1250.0 && cvars["meff"] < 1500.0;
  csel["SRC_bin4"] = csel["SRC"] && cvars["meff"] >= 1500.0 && cvars["meff"] < 1750.0;
  csel["SRC_bin5"] = csel["SRC"] && cvars["meff"] >= 1750.0 && cvars["meff"] < 2000.0;
  csel["SRC_bin6"] = csel["SRC"] && cvars["meff"] >= 2000.0 ;

}

bool tcMETMSSelection(std::map<std::string,float> cvars) {

  int mc_channel = int(cvars["mc_channel"]);

  if( (mc_channel==410470 || mc_channel==410646 || mc_channel==410647 || mc_channel==410654 || mc_channel==410655) && cvars["gen_met"]>200.) return false; //remove ttbar inc vs MET200 overlap

  //generic loose tcmet selection
  bool passtcMeTSelection = cvars["nbjets"]>=1 && cvars["ncjets"]>=1 && cvars["metOrig"]>250 && cvars["pT_1jet"]>50;

  //do a looser veto to see if baseline veto is the best thing for the SRC
  bool passSelection0L = (cvars["nsigLep"] == 0  && cvars["nbaselineLep"] == 0 &&  ( passtcMeTSelection));
  bool passSelection1L = (cvars["nsigLep"] == 1  && cvars["nbaselineLep"] == 1 &&  ( passtcMeTSelection));
  bool passSelection2L = (cvars["nsigLep"] == 2  && cvars["nbaselineLep"] == 2 &&  ( int(cvars["nsigMu"])==2 || int(cvars["nsigEl"])==2) && ( passtcMeTSelection)) && ( cvars["mll"] > 70 && cvars["mll"] < 110 && cvars["metOrig"] < 110 );

  //total selection
  bool passSelection = ( passSelection0L ||  passSelection1L || passSelection2L);

  return passSelection;

}

void tcMeT_2018::ProcessEvent(AnalysisEvent *event)
{

  //This is only setup for truth level analyses WITHOUT smearing.
  //Do NOT use this for reco level analysis or truth + smearing - the main issue is that getJets currently works for calorimeter jets only and needs to
  //be migrated to pflow jets. But also some WP such as DL1r for b-tagging also do not exist, as well as newer lepton WP etc.

  auto electrons  = event->getElectrons(7, 2.47, ELooseBLLH);
  auto muons      = event->getMuons(6, 2.7, MuMedium);

  float jetPtCut = 20.;
  float jetEtaCut = 2.8;
  auto candJets   = event->getJets(jetPtCut, jetEtaCut,PFlowJet);
  auto metVec     = event->getMET();
  //auto fatJets     = event->getFatJets(200.0,2.2,GoodFatJet);
  auto fatJets     = event->getFatJets(200.0,2.2); //,GoodFatJet);
  double met      = metVec.Et();
  double sumOfAllWeights = 0;
 
  for(unsigned int iw=0; iw < event->getMCWeights().size(); iw++){
    ntupVar(("mc_weight"+std::to_string(iw)).c_str(), event->getMCWeights()[iw]);
  }
  sumOfAllWeights += event->getMCWeights()[0];

  //for(unsigned int iw=0; iw < event->getMCWeights().size(); iw++){
  //  ntupVar(("mc_weight"+std::to_string(iw)).c_str(), event->getMCWeights()[iw]);
  //  sumOfAllWeights += event->getMCWeights()[iw];
  //}
  ntupVar("sumOfAllWeights",sumOfAllWeights);
  ntupVar("mcDSID", event->getMCNumber());

  //Jet Cleaning must use calorimeter jets, not pflow jets.
  auto caloCandJets = event->getJets(jetPtCut,jetEtaCut);
  if (countObjects(caloCandJets, 20, 2.8, NOT(LooseBadJet))!=0) return;

  //but the analysis uses pflow jets, so switch back to pflow the rest of the time
  candJets = filterObjects(candJets, 20, 2.8, PFlowJet);
  //candJets = filterObjects(candJets, 20, 2.8, JVT50Jet);

  // Overlap removal
  // not doing overlap removal between electrons and muons with identical track

  auto radiusCalcLepton = [] (const AnalysisObject& lepton, const AnalysisObject& ) { return std::min(0.4, 0.04 + 10/lepton.Pt()); };

  candJets   = overlapRemoval(candJets,electrons,0.2,NOT(BTag77DL1r));
  electrons  = overlapRemoval(electrons,candJets,0.4);
  candJets   = overlapRemoval(candJets, muons, 0.2, NOT(BTag77DL1r));
  muons      = overlapRemoval(muons, candJets, radiusCalcLepton);

  auto baselineLeptons   = electrons + muons;

  auto signalJets20    = filterObjects(candJets, 20);
  auto signalJets35    = filterObjects(candJets, 35);
  //Note that SUSYTools remaps egamma WP names such that TightLHElectron becomes TightLLH in the SUSY world
  //Thus we ask for ETightLH which is ETightLLH in the SUSY world. See both:
  //https://gitlab.cern.ch/atlas/athena/-/blob/21.2/PhysicsAnalysis/SUSYPhys/SUSYTools/Root/SUSYObjDef_xAOD.cxx#L1026
  //https://acode-browser1.usatlas.bnl.gov/lxr/source/athena/PhysicsAnalysis/ElectronPhotonID/ElectronPhotonSelectorTools/Root/EGSelectorConfigurationMapping.h
  auto signalElectrons = filterObjects(electrons, 20, 2.47, ETightLH | EIsoFCTight);
  auto signalMuons     = filterObjects(muons, 20, 2.5, MuMedium | MuIsoFCTightTrackOnly);
  auto signalLeptons   = signalElectrons + signalMuons;
  auto JetLep	       = filterObjects(signalLeptons + signalJets20, jetPtCut, jetEtaCut);

  auto signalNonBjets20   = filterObjects(signalJets20, 20., 2.8, NOT(BTag77DL1r)); //// ******** Pay attention to this implementation

  auto signalCjets20   = filterObjects(signalNonBjets20, 20., 2.5, TrueCJet);
  auto signalBjets20   = filterObjects(signalJets20, 20., 2.5, BTag77DL1r);
  auto signalBjets35   = filterObjects(signalJets35, 35., 2.5, BTag77DL1r);


  //auto topJets   = event->getHSTruth(350., 2.2,6);

  auto topJets   = filterObjects(fatJets, 350., 2.0, TopTag);

  AnalysisObjects fatJetsbtagged;
  AnalysisObjects fatJetsbtaggednonctagged;
  AnalysisObjects fatJetsttagged;
  
  for(auto fatjet: fatJets){
    bool btagged = false;
    bool ctagged = false;
    bool ttagged = false;
    for(auto bjet : signalBjets20){
      if(bjet.DeltaR(fatjet) < 1.0 ){
	btagged=true;
	break;
      }
    }
    for(auto cjet : signalCjets20){
      if(cjet.DeltaR(fatjet) < 1.0 ){
	ctagged=true;
	break;
      }
    }
    for(auto tjet : topJets){
      if(tjet.DeltaR(fatjet) < 1.0 ){
        ttagged=true;
        break;
      }
    }
    
    if(btagged and !ctagged)
      fatJetsbtaggednonctagged.push_back(fatjet);
    else if(btagged)
      fatJetsbtagged.push_back(fatjet);
    else if(ttagged)
      fatJetsttagged.push_back(fatjet);
  }



  //Fill in optional ntuple variables

  /*ntupVar("bj_N20",(int)signalBjets20.size());
  ntupVar("bj_N35",(int)signalBjets35.size());
  ntupVar("j_N20",(int)signalJets20.size());
  ntupVar("j_N35",(int)signalJets35.size());
  ntupVar("e_N",(int)electrons.size());
  ntupVar("m_N",(int)muons.size());
  ntupVar("jets",candJets);
*/

  //Common Selection
  int nJets20  = signalJets20.size();
  int nFatJets  = fatJets.size();
  int nBjets20 = signalBjets20.size();
  int nCjets20 = signalCjets20.size();

  int nTop = topJets.size();

  accept("SRAll"); //dummy selection to save variables/histos for the full sample

  //fill after OR
 // fill("MET",met);

  std::map<std::string,AnalysisObjects> evt_objs;
  evt_objs["BaseElectrons"] = electrons;
  evt_objs["BaseMuons"] = muons;
  evt_objs["SignalElectrons"] = signalElectrons;
  evt_objs["SignalMuons"] = signalMuons;
  evt_objs["jets35"] = signalJets35;
  evt_objs["jets"] = signalJets20;
  evt_objs["bjets35"] = signalBjets35;
  evt_objs["bjets"] = signalBjets20;
  evt_objs["nonbjets"] = signalNonBjets20;
  evt_objs["cjets"] = signalCjets20;
  evt_objs["fatjets"] = fatJets;
  evt_objs["topjets"] = topJets;
  evt_objs["fatjetsbtagged"] = fatJetsbtagged;
  evt_objs["fatjetsbtaggednonctagged"] = fatJetsbtaggednonctagged;
  evt_objs["fatjetsttagged"] = fatJetsttagged;
  evt_objs["JetLep"] = JetLep;

  std::map<std::string,float> common_variables;
  tcMETCommonVariables(common_variables,*event,*this,evt_objs);
  
  common_variables["nFatJets"] = nFatJets;
  common_variables["nTop"] = nTop;
  //super-common pre-selection
  if (nJets20<2)  return;
  //if (nBjets20<1) return;

  std::vector<float> input_values = {
    common_variables["mct2b"],common_variables["MTjmin"],common_variables["met"],common_variables["pT_1jet"],common_variables["pT_2jet"],common_variables["pT_3jet"],common_variables["pT_4jet"],
    common_variables["leadb1"],common_variables["leadb2"],common_variables["pT_1bjet"],common_variables["pT_2bjet"],common_variables["njets"],common_variables["metSigST"],
    common_variables["eta_1jet"],common_variables["eta_2jet"],common_variables["eta_3jet"],common_variables["eta_4jet"],common_variables["meff"],common_variables["dRb1b2"],
    common_variables["new_phib1_2ls"],common_variables["new_phib2_2ls"],common_variables["new_phij1_2ls"],common_variables["new_phij2_2ls"],common_variables["new_phij3_2ls"],common_variables["new_phij4_2ls"]
  };

  common_variables["metsigST"]=common_variables["metSigST"];

  // General preselections   
  std::map<std::string,bool> cSel;
  tcMETCommonSelections(cSel,common_variables);

  bool passSUSY1 = tcMETSUSY1Selection(*event,*this);
  bool passMasterShefSel = tcMETMSSelection(common_variables);

  //if ( !passSUSY1  ||  !passMasterShefSel )
  //return;

  // Variables transformed for the XGB

  bool passed = passSUSY1 && passMasterShefSel;
  ntupVar("passPresel",passed);
  bool passedSUSY = passSUSY1;
  //  ntupVar("passSUSY",passedSUSY);
  bool passedMS = passMasterShefSel;
  //  ntupVar("passMasterShef",passedMS);

  // tcMeT SRA
  tcMETSRASelections(common_variables,cSel);
  if( passed && cSel["SRA"] )
    accept("SRA");
  if( passed && cSel["SRA_bin1"] )
    accept("SRA_bin1");
  if( passed && cSel["SRA_bin2"] )
    accept("SRA_bin2");
  if( passed && cSel["SRA-CRZ"] )
    accept("SRA_CRZ");
  if( passed && cSel["SRA-CRst"] )
    accept("SRA_CRst");

  // tcMeT SRB

  tcMETSRBSelections(common_variables,cSel);
  if( passed && cSel["SRB0"] )
    accept("SRB0");
  if( passed && cSel["SRB0_bin1"] )
    accept("SRB0_bin1");
  if( passed && cSel["SRB0_bin2"] )
    accept("SRB0_bin2");
  if( passed && cSel["SRB0_bin3"] )
    accept("SRB0_bin3");

  if( passed && cSel["SRB1"] )
    accept("SRB1");
  if( passed && cSel["SRB1_bin1"] )
    accept("SRB1_bin1");
  if( passed && cSel["SRB1_bin2"] )
    accept("SRB1_bin2");
  if( passed && cSel["SRB1_bin3"] )
    accept("SRB1_bin3");
  if( passed && cSel["SRB1_bin4"] )
    accept("SRB1_bin4");


  // tcMeT SRC
  tcMETSRCSelections(common_variables,cSel);
  if( passed && cSel["SRC"] )
    accept("SRC");
  if( passed && cSel["SRC_bin1"] )
    accept("SRC_bin1");
  if( passed && cSel["SRC_bin2"] )
    accept("SRC_bin2");
  if( passed && cSel["SRC_bin3"] )
    accept("SRC_bin3");
  if( passed && cSel["SRC_bin4"] )
    accept("SRC_bin4");
  if( passed && cSel["SRC_bin5"] )
    accept("SRC_bin5");
  if( passed && cSel["SRC_bin6"] )
    accept("SRC_bin6");


  // ************************************
  // CutFlow section
  // ************************************

  // ************************************
  // End of cutflow section
  // ************************************


  for( auto sample  : common_variables)
    ntupVar(sample.first,sample.second);

  return;
}
